# -*- coding: utf-8 -*-
from icemac.ab.importer.reader.base import BaseReader
from mock import patch
import icemac.ab.importer.interfaces
import pytest
import zope.interface.verify


def test_base__BaseReader__1():
    """It fulfills the `IImportFileReader` interface."""
    assert zope.interface.verify.verifyObject(
        icemac.ab.importer.interfaces.IImportFileReader, BaseReader())


def test_base__BaseReader__canRead__1():
    """It returns `False` on an exception during calling `open()`."""
    with patch.object(BaseReader, 'open', side_effect=Exception):
        assert False is BaseReader.canRead(None)


def test_base__BaseReader__canRead__2():
    """It returns `False` on an exception during calling `getFieldNames()`."""
    with patch.object(BaseReader, 'getFieldNames', side_effect=Exception):
        assert False is BaseReader.canRead(None)


def test_base__BaseReader__canRead__3():
    """It returns `False` if `getFieldNames()` returns an empty list."""
    with patch.object(BaseReader, 'getFieldNames', return_value=[]):
        assert False is BaseReader.canRead(None)


def test_base__BaseReader__canRead__4():
    """It returns `True` if `getFieldNames()` returns an non-empty list."""
    with patch.object(BaseReader, 'getFieldNames', return_value=['sdfg']):
        assert True is BaseReader.canRead(None)


def test_base__BaseReader__getFieldNames__1():
    """It is not implemented."""
    with pytest.raises(NotImplementedError):
        BaseReader().getFieldNames()


def test_base__BaseReader__getFieldSamples__1():
    """It is not implemented."""
    with pytest.raises(NotImplementedError):
        BaseReader().getFieldSamples(None)


def test_base__BaseReader____iter____1():
    """It is not implemented."""
    with pytest.raises(NotImplementedError):
        BaseReader().__iter__()



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import datetime
import decimal
import gocept.reference.field
import icemac.ab.importer.browser.wizard.base
import icemac.ab.importer.interfaces
import icemac.addressbook.browser.errormessage
import icemac.addressbook.browser.interfaces
import icemac.addressbook.interfaces
import persistent.mapping
import time
import z3c.form.field
import zc.sourcefactory.contextual
import zope.component
import zope.interface
import zope.schema
import zope.security.proxy


NONE_REPLACEMENT = object()
TRUE_VALUES = ['yes', 'true']
FALSE_VALUES = ['no', 'false']
DATETIME_FORMAT = '%Y-%m-%d %H:%M'


def get_reader(session):
    if not hasattr(session, 'cache'):
        # we are in a dict below the session (storing field group
        # data), so we have to use the "uplink"
        session = session['__parent__']
    reader = session.cache.get('reader', None)
    if reader is not None:
        return reader
    reader_name = session['reader']
    reader = zope.component.getAdapter(
        None,
        icemac.ab.importer.interfaces.IImportFileReader,
        name=reader_name)
    file = zope.security.proxy.removeSecurityProxy(session.file.openDetached())
    reader = reader.open(file)
    session.cache['reader'] = reader
    return reader


class ImportFields(zc.sourcefactory.contextual.BasicContextualSourceFactory):
    """Where to put the imported data in the addressbook."""

    def getValues(self, context):
        return xrange(len(list(get_reader(context).getFieldNames())))

    def getTitle(self, context, value):
        reader = get_reader(context)
        field_name = list(reader.getFieldNames())[value]
        samples = u', '.join(
            x for x in reader.getFieldSamples(field_name) if x)
        title = field_name
        if samples:
            title = '%s (%s)' % (field_name, samples)
        return title


import_fields = ImportFields()


def split_keywords(keywords):
    return [x.strip() for x in keywords.split(',')]


class IFieldValue(zope.interface.Interface):
    """Value for a specific field."""

    def __call__():
        """Return the value."""


# field value converters

@zope.component.adapter(None, None)
@zope.interface.implementer(IFieldValue)
def unchanged_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value


@zope.interface.implementer(IFieldValue)
def text_field(value, field):
    """Adapter for `Text`, `TextLine` and `Choice` fields."""
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value.strip()


@zope.component.adapter(None, zope.schema.interfaces.IInt)
@zope.interface.implementer(IFieldValue)
def int_field(value, field):
    "Adapter for `Int` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        return int(float(value))
    except (TypeError, ValueError):
        return value


@zope.component.adapter(None, zope.schema.interfaces.IDecimal)
@zope.interface.implementer(IFieldValue)
def decimal_field(value, field):
    "Adapter for `Decimal` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        return decimal.Decimal(value)
    except (TypeError, ValueError, decimal.InvalidOperation):
        return value


@zope.component.adapter(None, zope.schema.interfaces.IBool)
@zope.interface.implementer(IFieldValue)
def bool_field(value, field):
    "Adapter for `Bool` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    if not isinstance(value, unicode):
        return value  # produces an error in validation
    value = value.lower()
    if value in TRUE_VALUES:
        return True
    if value in FALSE_VALUES:
        return False
    return value


@zope.component.adapter(None, zope.schema.interfaces.IDatetime)
@zope.interface.implementer(IFieldValue)
def datetime_field(value, field):
    "Adapter for `Datetime` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        time_tuple = time.strptime(value.strip(), DATETIME_FORMAT)
    except (TypeError, ValueError):
        return value
    else:
        return datetime.datetime(*time_tuple[:5])


@zope.component.adapter(None, zope.schema.interfaces.IURI)
@zope.interface.implementer(IFieldValue)
def uri_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value.strip().encode('ascii')


@zope.component.adapter(None, zope.schema.interfaces.IChoice)
@zope.interface.implementer(IFieldValue)
def country_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    value = value.strip()
    for candidate in field.source.factory.getValues():
        if candidate.token == value:
            return candidate
        if field.source.factory.getTitle(candidate) == value:
            return candidate
    # value not found in source --> can't adapt
    return None


@zope.component.adapter(None, gocept.reference.field.Set)
@zope.interface.implementer(IFieldValue)
def keywords_field(value, field):
    keywords = set()
    if value is None:
        return keywords
    keyword_util = zope.component.getUtility(
        icemac.addressbook.interfaces.IKeywords)
    for keyword_title in split_keywords(value):
        keywords.add(keyword_util.get_keyword_by_title(keyword_title))
    return keywords


# Error renderers

@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def simple_invalid(field, exc):
    return exc.doc()


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.schema.interfaces.ConstraintNotSatisfied)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def choice_constraint_not_satisfield(field, exc):
    value = exc.args[0]
    allowed = [str(x) for x in field.source.factory.getValues()]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value, values=', '.join(allowed)))


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.schema.interfaces.ConstraintNotSatisfied)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def country_constraint_not_satisfield(field, exc):
    value = exc.args[0]
    titles = [x.token for x in field.source.factory.getValues()]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value, values=', '.join(titles)))


@zope.component.adapter(zope.schema.interfaces.IDate,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def date_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is no valid date.', mapping=dict(value=value))


@zope.component.adapter(zope.schema.interfaces.IDatetime,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def datetime_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is no valid datetime. '
             u'Must match to format string "${format}".',
             mapping=dict(value=value, format=DATETIME_FORMAT))


@zope.component.adapter(zope.schema.interfaces.IInt,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def int_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is not a valid integer number.',
             mapping=dict(value=value))


@zope.component.adapter(zope.schema.interfaces.IDecimal,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def decimal_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is not a valid decimal number.',
             mapping=dict(value=value))


@zope.component.adapter(zope.schema.interfaces.IBool,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def bool_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value,
                          values=', '.join(TRUE_VALUES + FALSE_VALUES)))


@zope.component.adapter(None, IndexError)
@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def index_error(field, exc):
    return _(u'Not enough data fields in row.')


def render_error(entity, field_name, exc):
    "Render the error text using the error render adapters."
    obj_title = entity.title

    if field_name:
        field = entity.getField(field_name)
        title = _('${prefix} -- ${title}',
                  mapping=dict(prefix=obj_title, title=field.title))
    else:
        title = obj_title

    message = icemac.addressbook.browser.errormessage.render_error(
        entity, field_name, exc)

    # The rendered errors are stored in a set, so put title and
    # message here, as the message id is equal for all errors.
    mapping = (('title', title), ('message', message))
    return (mapping,
            _(u'${title}: ${message}', mapping=dict(mapping)))


class KeywordBuilder(object):

    KEYWORD_FIELD_NAME = "IcemacAddressbookPersonPerson-0.keywords"
    keyword_index = None
    keywords = None

    def __init__(self, user_data):
        """Expects a mapping between name of the field in the address book and
        field index in import file.

        Example: IcemacAddressbookPersonPerson-0.first_name --> 0
                 IcemacAddressbookAddressHomePageAddress-2.notes --> 11
        """
        self.keyword_index = user_data[self.KEYWORD_FIELD_NAME]
        self.keywords = zope.component.getUtility(
            icemac.addressbook.interfaces.IKeywords)

    def create(self, data):
        created = []
        if self.keyword_index is None:
            return created
        keyword_data = data[self.keyword_index]
        if keyword_data is None:
            return created
        for keyword_title in split_keywords(keyword_data):
            if self.keywords.get_keyword_by_title(keyword_title, None) is None:
                keyword = icemac.addressbook.utils.create_obj(
                    icemac.addressbook.keyword.Keyword, keyword_title)
                icemac.addressbook.utils.add(self.keywords, keyword)
                created.append(keyword)
        return created


class ImportObjectBuilder(object):
    """Build the objects the user wants to import."""

    def __init__(self, user_data, address_book, entries_number):
        """Expects a mapping between name of the field in the address book and
        field index in import file and
        .

        Example: IcemacAddressbookPersonPerson-0.first_name --> 0
                 IcemacAddressbookAddressHomePageAddress-3.notes --> 11

        Stores the address book field names in dictionaries on
        attributes mapping to the index in the import file.

        address_book ... address book to create the objects in.
        entries_number ... number of entries of each address kind.

        """
        self.address_book = address_book
        self.entries_number = entries_number
        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        self.import_entities = entities.getMainEntities()
        for entity in self.import_entities:
            for index in xrange(self.entries_number):
                key = "%s-%s" % (entity.name, index)
                setattr(self, key, {})
        for field_desc, index in user_data.iteritems():
            if index is None:
                continue  # field was not selected for import
            prefix, field_name = field_desc.split('.')
            getattr(self, prefix)[field_name] = index

    def create(self, data):
        """Create an object for data.

        data ... import data row, mapping between field index and value."""
        self.errors = set()
        person_entity = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntity,
            name='icemac.addressbook.person.Person')
        person = self._create('%s-0' % person_entity.name, self.address_book,
                              data, True)
        self._validate(person_entity, person)
        for entity in self.import_entities:
            if entity.class_name == 'icemac.addressbook.person.Person':
                # already handled
                continue
            for index in xrange(self.entries_number):
                prefix = "%s-%s" % (entity.name, index)
                main_entry = (index == 0)
                obj = self._create(prefix, person, data, main_entry)
                if main_entry:
                    # set the created address as main address of its kind
                    icemac.addressbook.person.get_default_field(
                        entity.interface).set(person, obj)
                if obj is not None:
                    self._validate(entity, obj)
        # self.errors contains the mapping and a message id, so sort
        # by mapping but return only the message id
        errors = sorted(list(self.errors), key=lambda x: x[0])
        return person, [x[1] for x in errors]

    def _create(self, prefix, parent, data, creation_required):
        field_mapping = getattr(self, prefix)
        len_data = len(data)
        field_values = [data[index]
                        for index in field_mapping.values()
                        if index < len_data and data[index]]
        if not (field_values or creation_required):
            # When there are no values to be stored and creation is
            # not required do nothing.
            return

        # instantiate the object
        entity_name, row_index = prefix.split('-')
        entity = icemac.addressbook.interfaces.IEntity(entity_name)
        obj = icemac.addressbook.utils.create_obj(entity.getClass())

        # set the values
        for field_name, index in field_mapping.iteritems():
            field = entity.getField(field_name)
            try:
                value = data[index]
            except IndexError, e:
                self.errors.add(render_error(entity, '', e))
            else:
                # try named converter first
                conv_value = zope.component.queryMultiAdapter(
                    (value, field), IFieldValue, name=field_name)
                if conv_value is None:
                    # fallback to default (unnamed) converter
                    conv_value = zope.component.getMultiAdapter(
                        (value, field), IFieldValue)
                if conv_value is NONE_REPLACEMENT:
                    conv_value = None
                context = field.interface(obj)
                try:
                    field.set(context, conv_value)
                except zope.interface.Invalid, e:
                    self.errors.add(render_error(entity, field_name, e))

        icemac.addressbook.utils.add(parent, obj)
        return obj

    def _validate(self, entity, obj):
        for field_name, field in entity.getFields():
            context = field.interface(obj)
            field = field.bind(context)
            value = field.get(context)
            try:
                field.validate(value)
            except zope.schema.ValidationError, exc:
                self.errors.add(render_error(entity, field_name, exc))


class FieldsGroup(z3c.form.group.Group):
    "Fields grouped by object."

    def __init__(self, context, request, parent, entity, label, prefix):
        super(FieldsGroup, self).__init__(context, request, parent)
        self.label = label
        self.prefix = prefix
        fields = []
        address_book = icemac.addressbook.interfaces.IAddressBook(None)
        customization = icemac.addressbook.interfaces.IFieldCustomization(
            address_book)

        for field_name, field in entity.getFields():
            field_title = customization.query_value(field, u'label')
            choice = zope.schema.Choice(
                title=field_title, description=field.description,
                required=False, source=import_fields)
            choice.__name__ = field_name
            fields.append(choice)
        self.fields = z3c.form.field.Fields(*fields, **dict(prefix=prefix))

    def getContent(self):
        if self.prefix not in self.context:
            self.context[self.prefix] = persistent.mapping.PersistentMapping()
            # We need to store the parent here to find the way back to
            # the parent in `get_reader`.
            self.context[self.prefix]['__parent__'] = self.context
        return self.context[self.prefix]


class MapFields(z3c.form.group.GroupForm,
                icemac.ab.importer.browser.wizard.base.FileSessionStorageStep):
    "Map the fields in the import file to fields in the addressbook."

    title = label = _(u'Map fields')

    def __init__(self, *args, **kw):
        super(MapFields, self).__init__(*args, **kw)
        session = self.getContent()
        request = self.request
        self.groups = []
        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        for entity in entities.getMainEntities():
            if entity.class_name == 'icemac.addressbook.person.Person':
                entries_number = 1
                main_prefix = ''
            else:
                entries_number = session.get('entries_number', 0)
                main_prefix = _(u'main')
            for index in xrange(entries_number):
                if index == 0:
                    row_title_prefix = main_prefix
                else:
                    row_title_prefix = _(u'other')
                title = _(
                    '${prefix} ${title}',
                    mapping=dict(prefix=row_title_prefix, title=entity.title))
                prefix = '%s-%s' % (entity.name, index)
                self.groups.append(
                    FieldsGroup(session, request, self, entity, title, prefix))

    @property
    def fields(self):
        return z3c.form.field.Fields()

    def update(self):
        super(MapFields, self).update()
        icemac.ab.importer.browser.wizard.base.FileSessionStorageStep.update(
            self)
        if not self.nextURL:
            # no redirect to next step, make sure no imported data exists
            icemac.ab.importer.browser.wizard.base.delete_imported_data(self)

    def applyChanges(self, data):
        super(MapFields, self).applyChanges(data)
        import_object_builder = ImportObjectBuilder(
            data, icemac.addressbook.interfaces.IAddressBook(self.context),
            self.getContent()['entries_number'])
        keyword_builder = KeywordBuilder(data)
        session = self.getContent()
        reader = get_reader(session)
        imported = session['imported'] = []
        imported_keywords = session['imported_keywords'] = []
        import_errors = session['import_errors'] = {}
        session['found_errors'] = False
        for data_row in reader:
            imported_keywords.extend(x.__name__
                                     for x in keyword_builder.create(data_row))
            obj, errors = import_object_builder.create(data_row)
            imported.append(obj.__name__)
            import_errors[obj.__name__] = errors
            if errors:
                session['found_errors'] = True



import icemac.ab.importer.importer
import icemac.ab.importer.interfaces
import icemac.addressbook.addressbook
import zope.component


@zope.component.adapter(icemac.addressbook.addressbook.AddressBookCreated)
def install_importer(event):
    """Install the importer in the newly created addressbook."""
    address_book = event.address_book
    icemac.addressbook.addressbook.create_and_register(
        address_book, 'importer', icemac.ab.importer.importer.Importer,
        icemac.ab.importer.interfaces.IImporter)



# -*- coding: utf-8 -*-
from decimal import Decimal
from icemac.addressbook.entities import get_bound_schema_field
from icemac.addressbook.interfaces import IEMailAddress
from icemac.addressbook.interfaces import IKeywords, IEntity, IPerson
from icemac.addressbook.interfaces import IPostalAddress, IPhoneNumber
import datetime
import icemac.addressbook.interfaces
import zope.component
import zope.component.hooks


def fields_and_values(person):
    """Get the field names and values of all non-empty person fields.

    The `country` field is omitted.
    """
    def entity_values(iface):
        entity = IEntity(iface)
        for name, field in entity.getRawFields():
            bound = get_bound_schema_field(person, entity, field)
            value = bound.get(bound.context)
            if value and field.title != 'country':
                yield field.title, value
    for iface in (IPerson, IPostalAddress, IPhoneNumber, IEMailAddress):
        for title, value in entity_values(iface):
            yield title, value


def test_map__1(address_book, browser, KeywordFactory, ImportFileFactory):
    """It imports keywords.

    Keywords are comma separated values inside a column. Not yet existing
    keywords get created. Existing ones get used.

    """
    # Create a keyword first to show it gets used:
    KeywordFactory(address_book, u'family')
    ImportFileFactory(address_book, u'keyword.csv', [
        'last_name,xkeywords',
        'Lahn,"family, friends"',
        'Sieg," friends , company"',
        'Pech,'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = [
        'last_name (Lahn, Sieg, Pech)']
    browser.getControl('keywords').displayValue = [
        'xkeywords (family, friends, friends , company)']
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert [
        'Lahn',
        'family, friends',
        'DE',
        'Sieg',
        'company, friends',
        'DE',
        'Pech',
        'DE'] == browser.etree.xpath('//table/tbody/tr/td/text()')
    # The import needs to be completed to view the results:
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # The new keywords have been created:
    keywords = zope.component.getUtility(IKeywords, context=address_book)
    keyword_titles = sorted(x.title for x in keywords.get_keywords())
    assert ([u'company', u'family', u'friends'] == keyword_titles)
    # They have been associated to the persons:
    with zope.component.hooks.site(address_book):
        assert ([
            (u'Lahn', [u'family', u'friends']),
            (u'Sieg', [u'company', u'friends']),
            (u'Pech', [])] ==
            [(x.get_name(), sorted([y.title for y in x.keywords]))
             for x in address_book.values()])


def test_map__2(
        address_book, browser, some_user_defined_fields, ImportFileFactory):
    """It imports values for user defined fields."""
    # Floating point numbers which are imported into an integer field are
    # floored and converted to an integer number. This is tested in the last
    # two lines of the import file.
    ImportFileFactory(address_book, u'fine.csv', [
        'last_name,foto,meet,states,numbers,costs,mailbox',
        'Utzer,True,2009-10-31 11:31, Sachsen-Anhalt,3,4.3,'
        'I do not like long and boring mail box texts where people tell a '
        'whole story instead of saying: I am not here leave a message.',
        'User,Yes,, Sachsen,6.2,1e3,',
        'Nobody,False,,,5.7,,'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = ['last_name']
    browser.getControl('photo permission?').displayValue = ['foto']
    browser.getControl('last seen').displayValue = ['meet']
    browser.getControl('state').displayValue = ['states']
    browser.getControl('number of letters').displayValue = ['numbers']
    browser.getControl('cost per minute').displayValue = ['costs']
    browser.getControl('mail box text').displayValue = ['mailbox']
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    # The imported data is shown in the preview, the contents of the `mail
    # box text` column is abbreviated:
    assert [
        'Utzer',
        'True',
        '2009-10-31 11:31:00',
        'DE',
        '3',
        'Sachsen-Anhalt',
        '4.3',
        u'I do not like long and …',
        'User',
        'True',
        'DE',
        '6',
        'Sachsen',
        '1E+3',
        'Nobody',
        'False',
        'DE',
        '5'] == browser.etree.xpath('//table/tbody/tr/td/text()')
    browser.getControl('Next').click()
    assert 'Import complete' in browser.contents
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    zope.component.hooks.setSite(address_book)
    p1 = address_book['Person']
    assert ([
        (u'last name', u'Utzer'),
        (u'photo permission?', True),
        (u'last seen', datetime.datetime(2009, 10, 31, 11, 31)),
        (u'number of letters', 3),
        (u'state', u'Sachsen-Anhalt'),
        (u'cost per minute', Decimal('4.3')),
        (u'mail box text', u'I do not like long and boring mail box texts '
                           u'where people tell a whole story instead of '
                           u'saying: I am not here leave a message.')
    ] == list(fields_and_values(p1)))
    p2 = address_book['Person-2']
    assert ([
        (u'last name', u'User'),
        (u'photo permission?', True),
        (u'number of letters', 6),
        (u'state', u'Sachsen'),
        (u'cost per minute', Decimal('1E+3'))
    ] == list(fields_and_values(p2)))
    p3 = address_book['Person-3']
    assert ([
        (u'last name', u'Nobody'),
        (u'number of letters', 5)
    ] == list(fields_and_values(p3)))


def test_map__3(address_book, browser, ImportFileFactory):
    """There was a bug which occurred when doing the following steps:

    * importing persons
    * deleting all persons
    * importing the same import file as in the first step

    """
    ImportFileFactory(address_book, u'del.csv', ["last_name", "Vipraschil"])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = ['last_name']
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # Delete all persons in address book
    browser.open(browser.ADDRESS_BOOK_EDIT_URL)
    browser.getControl('Delete all persons in address book').click()
    assert browser.ADDRESS_BOOK_DELETE_PERSONS_URL == browser.url
    browser.getControl('Yes').click()
    assert 'Address book contents deleted.' == browser.message
    # When importing the same import file, previously an error occurred:
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    assert (['Current step:', 'Map fields'] ==
            browser.etree.xpath('//div[@class="currStep"]/span/text()'))


def test_map__4(address_book, browser, ImportFileFactory):
    """It respects the customization of pre-defined fields."""
    field = icemac.addressbook.interfaces.IPersonName['first_name']
    customization = icemac.addressbook.interfaces.IFieldCustomization(
        address_book)
    customization.set_value(
        field, u'label', u'first name (given name or Christian name)')

    ImportFileFactory(address_book, u'del.csv', ["last_name", "Vipraschil"])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    assert browser.getControl('first name (given name or Christian name)')



from __future__ import unicode_literals


def test_edit__EditFile__1(
        address_book, browser, import_file, ImportFileFactory):
    """It allows to replace the import file."""
    ImportFileFactory(address_book, '1.csv', 'last_name')
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Back').click()
    assert browser.IMPORTER_IMPORT_EDIT_URL == browser.url
    browser.getControl('file', index=1).add_file(
        import_file(b'first_name'), 'text/plain', 'header.csv')
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_READER_URL == browser.url
    # The original file was really replaced:
    browser.getControl('Back').click()
    assert browser.IMPORTER_IMPORT_EDIT_URL == browser.url
    assert 'header.csv' == browser.getControl('name').value
    browser.getLink('Download file').click()
    assert 'first_name' == browser.contents



def test_readers__Readers__1(address_book, browser):
    """It lists all registered import file readers."""
    browser.login('mgr')
    browser.open(browser.IMPORTER_OVERVIEW_URL)
    browser.getLink('Supported import file formats').click()
    assert browser.IMPORTER_READERS_LIST_URL == browser.url
    assert (['CSV file (comma separated fields, ISO-dates, UTF-8 encoded)'] ==
            browser.etree.xpath('//ul[@class="bullet"]/li/text()'))



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.ab.importer.browser.wizard.base
import icemac.ab.importer.source
import zope.interface
import zope.schema


class IReaderSettings(zope.interface.Interface):
    """Reader Setttings."""

    reader = zope.schema.Choice(
        title=_(u'Import file format'),
        description=_(
            "If the drop-down is empty the file's format is not supported."),
        source=icemac.ab.importer.source.Importers())

    entries_number = zope.schema.Int(
        title=_('person sub-entities'),
        description=_(
            'This is the maximum number per person of postal addresses, phone'
            ' numbers, e-mail addresses resp. homepage addresses the import'
            ' file contains.'),
        min=1, max=9, default=1)


class ReaderSettings(
        icemac.ab.importer.browser.wizard.base.FileSessionStorageStep):
    """Step to set the reader and the number of entries."""

    interface = IReaderSettings
    title = label = _(u'File format settings')

    @property
    def showBackButton(self):
        """Back button condition."""
        return True

    def doBack(self, action):
        """Process back action and return True on success."""
        self.getContent()['edit_file_available'] = True
        return super(ReaderSettings, self).doBack(action)



# -*- coding: utf-8 -*-
from icemac.ab.importer.interfaces import IImportFileReader
from icemac.ab.importer.source import Importers
import StringIO
import icemac.ab.importer.interfaces
import icemac.ab.importer.reader.base
import icemac.ab.importer.source
import pytest
import zope.component
import zope.component.globalregistry
import zope.interface
import zope.interface.verify


class DummyReader(icemac.ab.importer.reader.base.BaseReader):
    """Dummy reader implementation."""

    title = u'Dummy Reader'

    def getFieldNames(self):
        """Get the names of the fields in the file."""
        # canRead requires at least a field name to return True
        return ['field1']

    def getFieldSamples(self, field_name):
        """Get sample values for a field."""

    def __iter__(self):
        """Iterate over the file."""


class NoXMLReader(DummyReader):
    """A dummy reader implementation which does not like XML files."""

    def getFieldNames(self):
        self.file.seek(0)
        data = self.file.read()
        if data.startswith('<?xml'):
            raise ValueError('No XML!')
        return ['field1']


@zope.interface.implementer(icemac.ab.importer.interfaces.IImportFile)
class DummyImportFile(object):
    """A dummy import file implemention."""

    def __init__(self, data):
        self.data = data

    def openDetached(self):
        return StringIO.StringIO(self.data)


@pytest.yield_fixture('module', autouse=True)
def dummy_readers(zcmlS):
    """Register some dummy readers for the tests."""
    gsm = zope.component.globalregistry.getGlobalSiteManager()
    gsm.registerAdapter(DummyReader, name='dummy')
    gsm.registerAdapter(NoXMLReader, name='no-xml')
    yield
    gsm.unregisterAdapter(DummyReader, name='dummy')
    gsm.unregisterAdapter(NoXMLReader, name='no-xml')


def test_test_source__1():
    """Verify the dummy objects fulfilling thier interfaces."""
    assert zope.interface.verify.verifyObject(IImportFileReader, DummyReader())
    assert zope.interface.verify.verifyObject(IImportFileReader, NoXMLReader())


def test_source__Importers__getValue__1():
    """It returns all readers capable to read text files."""
    file = DummyImportFile('text')
    assert [u'dummy',
            u'csv-commaseparated-utf8-isodate',
            u'no-xml'] == list(Importers().factory.getValues(file))


def test_source__Importers__getValue__2():
    """It returns all readers capable to read XML files."""
    file = DummyImportFile('<?xml ...')
    assert ([u'dummy', u'csv-commaseparated-utf8-isodate'] ==
            list(Importers().factory.getValues(file)))


def test_source__Importers__getTitle__1():
    """It returns the title of the reader."""
    file = DummyImportFile('text')
    assert u'Dummy Reader' == Importers().factory.getTitle(file, 'dummy')
    assert u'Dummy Reader' == Importers().factory.getTitle(file, 'no-xml')



import pytest


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_base__ImportWizard__1(address_book, browser, role):
    """It is not allowed to be accessed by some user roles."""
    browser.login(role)
    browser.assert_forbidden(browser.IMPORTER_FILE_IMPORT_URL)



from icemac.addressbook.interfaces import IPhoneNumber
import zope.component.hooks


def test_reader__ReaderSettings__1(address_book, browser, ImportFileFactory):
    """It has an empty import file readers list on empty import file."""
    ImportFileFactory(address_book, u'empty.csv', [''])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    # There is no import file reader, as the CSV reader expects a first line
    # containing the field names:
    assert [] == browser.getControl('Import file format').displayOptions


def test_reader__ReaderSettings__2(address_book, browser, ImportFileFactory):
    """It allows to import multiple addresses and numbers."""
    ImportFileFactory(address_book, u'multi.csv', [
        'last_name,1st_phone,2nd_phone,3rd_phone',
        'One,112,110,108',
        'Two,,116116,',
        'Three,1111,,2222',
        'Four,,,'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    assert '1' == browser.getControl('person sub-entities').value
    browser.getControl('person sub-entities').value = '3'
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = [
        'last_name (One, Two, Three)']
    browser.getControl('number', index=0).displayValue = [
        '1st_phone (112, 1111)']
    browser.getControl('number', index=1).displayValue = [
        '2nd_phone (110, 116116)']
    browser.getControl('number', index=2).displayValue = [
        '3rd_phone (108, 2222)']
    browser.getControl('Next').click()
    # The review step shows the import result, the first phone number is the
    # default one:
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert [
        'first name',
        'last name',
        'birth date',
        'keywords',
        'notes',
        'address prefix',
        'street',
        'city',
        'zip',
        'country',
        'address prefix',
        'street',
        'city',
        'zip',
        'country',
        'address prefix',
        'street',
        'city',
        'zip',
        'country',
        'number',
        'number',
        'number',
        'e-mail address',
        'e-mail address',
        'e-mail address',
        'URL',
        'URL',
        'URL',
        'One',
        'DE',
        '112',
        '110',
        '108',
        'Two',
        'DE',
        '116116',
        'Three',
        'DE',
        '1111',
        '2222',
        'Four',
        'DE'] == browser.etree.xpath('//th/text() | //td//text()')
    # After completing the import the imported data can be inspected:
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    zope.component.hooks.setSite(address_book)
    # The first phone number is the main one and it is displayed as the main
    # phone number in the form below, too:
    p1 = address_book['Person']
    assert u'One' == p1.get_name()
    assert '112' == p1.default_phone_number.number
    assert (['112', '110', '108'] ==
            [x.number for x in p1.values() if IPhoneNumber.providedBy(x)])
    # When there is no first one an empty main number is created:
    p2 = address_book['Person-2']
    assert u'Two' == p2.get_name()
    assert None is p2.default_phone_number.number
    assert ([None, '116116'] ==
            [x.number for x in p2.values() if IPhoneNumber.providedBy(x)])
    # Empty non-main phone numbers are not created, so the missing 2nd number
    # is left out and the third one gets the one and only non-main phone
    # number:
    p3 = address_book['Person-3']
    assert u'Three' == p3.get_name()
    assert '1111' == p3.default_phone_number.number
    assert (['1111', '2222'] ==
            [x.number for x in p3.values() if IPhoneNumber.providedBy(x)])
    # But there must be a default number which is always created:
    p4 = address_book['Person-4']
    assert u'Four' == p4.get_name()
    assert None is p4.default_phone_number.number
    assert ([None] ==
            [x.number for x in p4.values() if IPhoneNumber.providedBy(x)])



# -*- coding: utf-8 -*-
from datetime import datetime
from icemac.ab.importer.browser.table import ModifiedColumnLocalTime
from pytz import utc
from zope.i18n.locales import LocaleTimeZone
from zope.publisher.browser import TestRequest as ExampleRequest


class DummyDC(object):
    """Dummy dublin core object."""

    modified = datetime(2003, 5, 3, 11, 12, 13, tzinfo=utc)


def test_table__ModifiedColumnLocalTime__getValue__1():
    """It returns an empty string for `None`."""
    column = ModifiedColumnLocalTime(None, ExampleRequest(), None)
    assert u'' == column.getValue(None)


def test_table__ModifiedColumnLocalTime__getValue__2():
    """It returns None if `modified` is `None`."""
    column = ModifiedColumnLocalTime(None, ExampleRequest(), None)
    dc = DummyDC()
    dc.modified = None
    assert column.getValue(dc) is None


def test_table__ModifiedColumnLocalTime__getValue__3():
    """It returns modified in UTC if nothing else is defined."""
    column = ModifiedColumnLocalTime(None, ExampleRequest(), None)
    dc = DummyDC()
    value = column.getValue(dc)
    assert dc.modified == value
    assert 'UTC' == value.tzinfo.zone


def test_table__ModifiedColumnLocalTime__getValue__4():
    """It returns modified in local timezone if it is set."""
    column = ModifiedColumnLocalTime(None, ExampleRequest(), None)
    try:
        column.request.locale.dates.timezones[u'Europe/Berlin'] = (
            LocaleTimeZone(u'Europe/Berlin'))
        dc = DummyDC()
        value = column.getValue(dc)
        assert dc.modified == value
        assert 'Europe/Berlin' == value.tzinfo.zone
    finally:
        # timezones seems to store information globally
        ExampleRequest().locale.dates.timezones.pop(u'Europe/Berlin', None)



import pytz
import z3c.table.column


class ModifiedColumnLocalTime(z3c.table.column.ModifiedColumn):
    """Modification date in local time of the browser."""

    def getValue(self, item):
        value = super(ModifiedColumnLocalTime, self).getValue(item)
        if value:
            timezones = self.request.locale.dates.timezones
            timezone = None
            for tz in timezones.values():
                if tz.type:
                    timezone = tz.type
            if timezone:
                timezone = pytz.timezone(timezone)
                value = value.astimezone(timezone)
        return value



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.ab.importer.interfaces
import icemac.addressbook.browser.base
import zope.component


class Readers(icemac.addressbook.browser.base.FlashView):
    """List the known readers."""

    title = _('Supported import file formats')

    def readers(self):
        readers = zope.component.getAdapters(
            (None, ), icemac.ab.importer.interfaces.IImportFileReader)
        for name, reader in readers:
            yield reader.title



from StringIO import StringIO
import pytest


@pytest.fixture('session')
def import_file():
    """Get a function to create a file like object for import."""
    def create_file(data):
        file = StringIO()
        file.write(data)
        file.seek(0)
        return file
    return create_file



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.wizard
import zope.traversing.api
import zope.traversing.browser.absoluteurl


class Complete(icemac.addressbook.browser.wizard.Step):
    """Step displaying that the import was completed."""

    title = label = _(u'Complete')
    fields = {}

    def applyChanges(self, data):
        super(Complete, self).applyChanges(data)
        self.wizard.nextURL = zope.traversing.browser.absoluteurl.absoluteURL(
            zope.traversing.api.getParent(self.context), self.request)



from icemac.addressbook.i18n import _
import icemac.ab.importer.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.browser.file.file


class Edit(icemac.addressbook.browser.base.BaseEditForm):
    """Edit an import file."""

    interface = icemac.ab.importer.interfaces.IImportFile
    title = _('Edit file')
    next_url = 'parent'

    def applyChanges(self, data):
        changes = super(Edit, self).applyChanges(data)
        icemac.addressbook.browser.file.file.update_blob(
            self.widgets['data'], self.context)
        return changes



import icemac.ab.importer.install
import icemac.addressbook.addressbook
import icemac.addressbook.generations.utils


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(address_book):
    """Install the importer into each existing address book."""
    icemac.ab.importer.install.install_importer(
        icemac.addressbook.addressbook.AddressBookCreated(address_book))



from __future__ import unicode_literals
import pytest


@pytest.fixture(scope='function')
def md_menu(address_book, browser, sitemenu):
    """Fixture to test the master data menu."""
    browser.login('mgr')
    return sitemenu(
        browser, 4, 'Master data', browser.IMPORTER_OVERVIEW_URL)


def test_menu__master_data_menu__1(md_menu):
    """The master data menu item is selected on the importer overview."""
    assert md_menu.item_selected(md_menu.menu_item_URL)


def test_menu__master_data_menu__2(md_menu, address_book, ImportFileFactory):
    """The master data menu item is selected on the import file."""
    ImportFileFactory(address_book, u'file.csv', ['file contents'])
    assert md_menu.item_selected(md_menu.browser.IMPORTER_FILE_EDIT_URL)



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.importer.browser.table
import icemac.ab.importer.interfaces
import icemac.addressbook.browser.breadcrumb
import icemac.addressbook.browser.file.file
import icemac.addressbook.browser.table
import z3c.table.column
import zope.security.proxy


class ImporterBreadCrumb(
        icemac.addressbook.browser.breadcrumb.MasterdataChildBreadcrumb):
    """Breadcrumb for the importer."""

    grok.adapts(
        icemac.ab.importer.interfaces.IImporter,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Importer')


class Overview(icemac.addressbook.browser.table.PageletTable):
    """List of already uploaded import files + action links."""

    title = _('Files for import')
    no_rows_message = _(u'No import files uploaded, yet.')

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'file', weight=1),
            z3c.table.column.addColumn(
                self, z3c.table.column.GetAttrColumn, 'mimeType', weight=2,
                header=_(u'MIME type'), attrName='mimeType'),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TruncatedContentColumn,
                'notes', weight=3, header=_(u'Notes'), attrName='notes',
                length=50),
            z3c.table.column.addColumn(
                self, icemac.ab.importer.browser.table.ModifiedColumnLocalTime,
                'modified', weight=5),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.DeleteLinkColumn,
                'delete', weigth=6),
            z3c.table.column.addColumn(
                self, z3c.table.column.LinkColumn, 'import', weight=200,
                header=_(u''), linkContent=_(u'Import'),
                linkName='@@import')]

    @property
    def values(self):
        return self.context.values()


def provide_IImportFile(file):
    """Provide the `IImportFile` marker interface."""
    zope.interface.directlyProvides(
        file, zope.security.proxy.removeSecurityProxy(
            icemac.ab.importer.interfaces.IImportFile))


class Add(icemac.addressbook.browser.file.file.Add):
    """Add a file with the `IImportFile` marker interface."""

    interface = icemac.ab.importer.interfaces.IImportFile

    def create(self, data):
        file = icemac.addressbook.utils.create_obj(self.class_)
        provide_IImportFile(file)
        icemac.addressbook.browser.file.file.update_blob(
            self.widgets['data'], file)
        return file



# -*- coding: utf-8 -*-
import os.path
import setuptools


def read(*path_elements):
    """Read a path."""
    return open(os.path.join(*path_elements)).read()

version = '2.11'
long_description = '\n\n'.join([read('README.rst'),
                                read('CHANGES.rst')])

setuptools.setup(
    name='icemac.ab.importer',
    version=version,
    description="Import infrastructure for icemac.addressbook",
    long_description=long_description,
    keywords='icemac.addressbook',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    url='https://pypi.org/project/icemac.ab.importer',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Natural Language :: German',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['icemac', 'icemac.ab'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'gocept.cache',
        'grokcore.component',
        'icemac.ab.locales >= 0.8',
        'icemac.addressbook >= 9.2',
        'icemac.truncatetext >= 0.2',
        'pytz',
        'setuptools',
        'z3c.wizard',
        'zc.sourcefactory',
        'zope.container',
        'zope.generations',
        'zope.interface',
        'zope.schema',
        'zope.securitypolicy >= 4.1',
    ],
    extras_require=dict(
        test=[
            'icemac.addressbook [test]',
            'zope.testing >= 3.8.0',
            'zope.traversing',
        ]),
)



import icemac.addressbook.testing


class Browser(icemac.addressbook.testing.Browser):
    """Browser adapted for importer."""

    IMPORTER_OVERVIEW_URL = 'http://localhost/ab/++attribute++importer'

    IMPORTER_FILE_ADD_URL = (
        'http://localhost/ab/++attribute++importer/@@addFile.html')
    IMPORTER_FILE_IMPORT_URL = (
        'http://localhost/ab/++attribute++importer/File/@@import')
    IMPORTER_IMPORT_EDIT_URL = (
        'http://localhost/ab/++attribute++importer/File/import/editFile')
    IMPORTER_IMPORT_READER_URL = (
        'http://localhost/ab/++attribute++importer/File/import/reader')
    IMPORTER_IMPORT_MAP_URL = (
        'http://localhost/ab/++attribute++importer/File/import/map')
    IMPORTER_IMPORT_REVIEW_URL = (
        'http://localhost/ab/++attribute++importer/File/import/review')
    IMPORTER_IMPORT_COMPLETE_URL = (
        'http://localhost/ab/++attribute++importer/File/import/complete')
    IMPORTER_FILE_EDIT_URL = 'http://localhost/ab/++attribute++importer/File'
    IMPORTER_FILE_DOWNLOAD_URL = (
        'http://localhost/ab/++attribute++importer/File/download.html')
    IMPORTER_FILE_DELETE_URL = (
        'http://localhost/ab/++attribute++importer/File/@@delete.html')

    IMPORTER_READERS_LIST_URL = (
        'http://localhost/ab/++attribute++importer/@@readers.html')



from icemac.ab.importer.browser.importer import provide_IImportFile
import icemac.ab.importer
import icemac.ab.importer.testing
import icemac.addressbook.testing
import pytest


pytest_plugins = 'icemac.addressbook.fixtures'

# Fixtures to set-up infrastructure which are usable in tests:


@pytest.yield_fixture(scope='function')
def address_book(addressBookConnectionF):
    """Get the address book with importer as site."""
    for address_book in icemac.addressbook.testing.site(
            addressBookConnectionF):
        yield address_book


@pytest.fixture(scope='function')
def browser(browserWsgiAppS):
    """Fixture for testing with zope.testbrowser."""
    assert icemac.addressbook.testing.CURRENT_CONNECTION is not None, \
        "The `browser` fixture needs a database fixture like `address_book`."
    return icemac.ab.importer.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.fixture(scope='session')
def ImportFileFactory(FileFactory):
    """Create an import file in the importer.

    data ... list of the lines in the import file.
    """
    def create_file(address_book, filename, data, **kw):
        file = FileFactory(
            address_book.importer, filename, data='\n'.join(data), **kw)
        provide_IImportFile(file)
        return file
    return create_file


# Fixtures to help asserting

@pytest.fixture(scope='function')
def sitemenu(browser):
    """Helper fixture to test the selections in the site menu."""
    return icemac.addressbook.testing.SiteMenu


@pytest.fixture(scope='function')
def assert_address_book(address_book):
    """Fixture returning an object providing a custom address book asserts."""
    return icemac.addressbook.testing.AddressBookAssertions(address_book)


# Infrastructure fixtures


@pytest.yield_fixture(scope='session')
def zcmlS():
    """Load importer ZCML on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'Importer', __name__, icemac.ab.importer)
    layer.setUp()
    yield layer
    layer.tearDown()


@pytest.yield_fixture(scope='session')
def zodbS(zcmlS):
    """Create an empty test ZODB."""
    for zodb in icemac.addressbook.testing.pyTestEmptyZodbFixture():
        yield zodb


@pytest.yield_fixture(scope='session')
def addressBookS(zcmlS, zodbS):
    """Create an address book for the session."""
    for zodb in icemac.addressbook.testing.pyTestAddressBookFixture(
            zodbS, 'ImporterS'):
        yield zodb


@pytest.yield_fixture(scope='function')
def addressBookConnectionF(addressBookS):
    """Get the connection to the right demo storage."""
    for connection in icemac.addressbook.testing.pyTestStackDemoStorage(
            addressBookS, 'ImporterF'):
        yield connection
